/*************************************************************

(C) ISEP 2004
(C) Professores de ADAV 2004

O ISEP, na qualidade de autor do presente software, garante 
o direito de utilizao, alterao, evoluo e distribuio 
do mesmo sem qualquer tipo de restries, excepto:
	- esta mensagem no pode ser retirada
	- os nomes das classes no podem ser alterados

*************************************************************/

using System;
using System.Data;
using System.Data.OleDb;

using Isep.Util.Data;

namespace ShopClassLibrary
{
	// nova interface criada para suportar web services
	public interface IOrderWeb
	{
		DataSet CreateDetails(string customerUserName, string customerPassword, out ShopStatusEnum status);
		ShopStatusEnum PlaceOrder(string customerUserName, string customerPassword, DateTime expectedDeliveryDate, DataSet details);
	}

	/// <summary>
	/// Summary description for Order.
	/// </summary>
	public class Order : IOrderWeb
	{
		public Order()
		{
			//
			// TODO: Add constructor logic here
			//
		}
		#region IOrderWeb Members

		public DataSet CreateDetails(string customerUserName, string customerPassword, out ShopStatusEnum status)
		{
			// TODO:  Add Order.CreateDetails implementation
			status = ShopStatusEnum.NotSupported;
			return null;
		}

		public ShopStatusEnum PlaceOrder(string customerUserName, string customerPassword, DateTime expectedDeliveryDate, DataSet details)
		{
			ShopStatusEnum status;

			//
			// validaes
			//
			if (customerUserName.Length == 0 || customerPassword.Length == 0)
				return ShopStatusEnum.InvalidArgument;
			if (details.Tables.Count != 1 || details.Tables[0].Columns.Count != 2 || details.Tables[0].Columns[0].ColumnName != "ProductID" || details.Tables[0].Columns[1].ColumnName != "Quantity")
				return ShopStatusEnum.InvalidArgument;

			//processamento
			OleDbConnection conn = null;
			OleDbTransaction tx = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				tx = OleDbHelper.CreateTransactedConnection(UtilDB.CONN);
				conn = tx.Connection;
				
				//
				// validar o utilizador
				//
				int customerID;
				status = UtilDB.ValidateCustomerLogin(conn, tx, customerUserName, customerPassword, out customerID);
				if (status != ShopStatusEnum.Ok)
					return status;

				status = ShopStatusEnum.Ok;
			
				//
				// validar se artigos existem
				//
				System.Text.StringBuilder sb = new System.Text.StringBuilder("SELECT count(*) FROM Products WHERE ProductId IN (");
				foreach (DataRow regDetails in details.Tables[0].Rows)
				{
					if (regDetails.IsNull("ProductID"))
						return ShopStatusEnum.InvalidArgument; //INVALID_PRODUCT_ID;
					if (regDetails.IsNull("Quantity"))
						return ShopStatusEnum.InvalidArgument;

					//
					// guardar o cdigo de produto para pesquisa
					//
					int k = (int)regDetails["ProductID"];
					sb.Append(k.ToString());
					sb.Append(",");
				}
				sb.Length = sb.Length-1;
				sb.Append(")");
				string sSqlCmd = sb.ToString();

				// verificar se os produtos existem em catalogo
				int n = (int)OleDbHelper.ExecuteScalar(tx, sSqlCmd);
				if (n != details.Tables[0].Rows.Count)
					return ShopStatusEnum.InvalidProductID;

				//
				// inserir cabealho da encomenda
				//
				object[] parms = new object[] {
												"@cid", OleDbType.Integer, customerID,
												"@dtp", OleDbType.Date, DateTime.Now,
												"@dtexp", OleDbType.Char, expectedDeliveryDate.ToShortDateString()
											  };
				int idOrder = OleDbHelper.InsertAndGetID(tx, "INSERT INTO Orders (CustomerID, datePlaced, dateExpectedDelivery) VALUES(?, ?, ?)", parms);

				//
				// inserir linhas de detalhe
				//
				OleDbCommand cmd = new OleDbCommand("INSERT INTO OrderDetails VALUES(?, ?, ?)", conn, tx);
				cmd.Parameters.Add("@oid", OleDbType.Integer);
				cmd.Parameters.Add("@pid", OleDbType.Integer);
				cmd.Parameters.Add("@qt", OleDbType.Integer);

				foreach (DataRow regDetails in details.Tables[0].Rows)
				{
					cmd.Parameters[0].Value = idOrder;
					cmd.Parameters[1].Value = regDetails["ProductID"];
					cmd.Parameters[2].Value = regDetails["Quantity"];

					cmd.ExecuteNonQuery();
				}

				tx.Commit();
			}
			catch(OleDbException ex)
			{
				if (tx != null)
					tx.Rollback();

				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <PlaceOrder> da classe 'Order': " + ex.Message);
				status = ShopStatusEnum.DatabaseError;
			}
			catch(Exception ex)
			{
				if (tx != null)
					tx.Rollback();

				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <PlaceOrder> da classe 'Order': " + ex.Message);
				status = ShopStatusEnum.Error;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		#endregion
	}
}
