/*************************************************************

(C) ISEP 2004
(C) Professores de ADAV 2004

O ISEP, na qualidade de autor do presente software, garante 
o direito de utilizao, alterao, evoluo e distribuio 
do mesmo sem qualquer tipo de restries, excepto:
	- esta mensagem no pode ser retirada
	- os nomes das classes no podem ser alterados

*************************************************************/

using System;
using System.Data;
using System.Data.OleDb;

using Isep.Util.Data;

namespace ShopClassLibrary
{
	public interface IProduct
	{
		DataSet GetByID(string User, string Pass, int ProductID);
		DataSet FindByName(string User, string Pass, string NamePattern);
		int GetStock(string User, string Pass, int ProductID, out ShopStatusEnum status);
		ShopStatusEnum InStock(string User, string Pass, int ProductID, int Quant);
		ShopStatusEnum AddStock(string User, string Pass, int ProductID, int Quant);
	}

	// nova interface criada para suportar web services
	public interface IProductWeb
	{
		DataSet GetCatalog(string customerUserName, string customerPassword, out ShopStatusEnum status);
	}

	/// <summary>
	/// Summary description for Product.
	/// </summary>
	public class Product : IProduct, IProductWeb
	{
		#region IProduct
		/// <summary>
		/// pesquisa um determinado produto por cdigo de produto
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>a informao do produto</returns>
		public DataSet GetByID(string User, string Pass, int ProductID)
		{
			OleDbConnection conn = null;
			DataSet rs = null;
			ShopStatusEnum status = ShopStatusEnum.Ok;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar utilizar
				//
				status = UtilDB.ValidateUser(conn, null, User, Pass);
				if (status != ShopStatusEnum.Ok)
					return null;

				//
				// efectuar pesquisa
				//
				rs = UtilDB.GetByID(conn, null, "Products", "ProductID", ProductID, out status);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetById> da classe 'Product': " + ex.Message);
				status = ShopStatusEnum.DatabaseError;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return rs;
		}

		/// <summary>
		/// pesquisa produtos por nome
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>informao dos produtos que obedeam ao critrio de busca</returns>
		public DataSet FindByName(string User, string Pass, string NamePattern)
		{
			OleDbConnection conn = null;
			DataSet rs = null;
			ShopStatusEnum status = ShopStatusEnum.Ok;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				if( UtilDB.ValidateUser(conn, null, User, Pass) != ShopStatusEnum.Ok )
					return null;

				//
				// efectuar a pesquisa
				//
				rs = UtilDB.FindByStringField(conn, "Products", "Description", NamePattern, out status);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <FindByName> da classe 'Product': " + ex.Message);
				 status = ShopStatusEnum.DatabaseError;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return rs;
		}

		/// <summary>
		/// devolve a quantidade em stock de um determinado produto
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="status"></param>
		/// <returns></returns>
		public int GetStock(string User, string Pass, int ProductID, out ShopStatusEnum status)
		{
			OleDbConnection conn = null;
			int qt;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, User, Pass);
				if (status != ShopStatusEnum.Ok)
					return -1;

				//
				// preparar comando
				//
				OleDbCommand cmd = new OleDbCommand("SELECT StockQtd FROM Products WHERE ProductId=?", conn);
				cmd.Parameters.Add("pid", ProductID);

				//
				// executar comando
				//
				qt = (int)cmd.ExecuteScalar();

				status = ShopStatusEnum.Ok;
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <InStock> da classe 'Product': " + ex.Message);
				status = ShopStatusEnum.DatabaseError;
				return -1;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return qt;
		}

		/// <summary>
		/// verifica se existe uma determinada quantidade de um produto em armazem 
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="Quant">quantidade desejada de produto</param>
		/// <returns>cdigo de estado relativo  operao</returns></returns></returns>
		public ShopStatusEnum InStock(string User, string Pass, int ProductID, int Quant)
		{
			OleDbConnection conn = null;
			ShopStatusEnum status;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, User, Pass);
				if (status != ShopStatusEnum.Ok)
					return status;

				//
				// invocar mtodo auxiliar
				//
				status = InStock(conn, null, ProductID, Quant);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <InStock> da classe 'Product': " + ex.Message);
				return ShopStatusEnum.DatabaseError; // ShopStatusEnum.NotOk;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}

		public ShopStatusEnum AddStock(string User, string Pass, int ProductID, int Quant)
		{
			OleDbConnection conn = null;
			ShopStatusEnum status;

			//
			// validar parametros de entrada
			//
			if (Quant <= 0)
				return ShopStatusEnum.InvalidArgument;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, User, Pass);
				if (status != ShopStatusEnum.Ok)
					return status;

				//
				// criar comando SQL a executar
				//
				string sSqlCmd = "Update Products SET StockQtd = StockQtd + ? WHERE ProductID = " + ProductID.ToString(); 
				OleDbCommand cmd = new OleDbCommand(sSqlCmd, conn);
				cmd.Parameters.Add("qt", Quant);

				//
				// executar o comando 
				//
				int updRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (updRows == 0)
					// no actualizou registo
					status = ShopStatusEnum.NotOk;
				else
					// actualizou registo
					status = ShopStatusEnum.Ok;
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <AddStock> class 'Product': " + ex.Message);
				status = ShopStatusEnum.DatabaseError;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return status;
		}
		#endregion

		#region IProductWeb Members

		public DataSet GetCatalog(string customerUserName, string customerPassword, out ShopStatusEnum status)
		{
			// validaes
			if (customerUserName.Length == 0 || customerPassword.Length == 0)
			{
				status = ShopStatusEnum.InvalidArgument;
				return null;
			}

			//processamento
			OleDbConnection conn = null;
			DataSet ds = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = OleDbHelper.CreateOpenConnection(UtilDB.CONN);
				
				//
				// validar o utilizador
				//
				int customerID;
				status = UtilDB.ValidateCustomerLogin(conn, null, customerUserName, customerPassword, out customerID);
				if (status != ShopStatusEnum.Ok)
					return null;

				status = ShopStatusEnum.Ok;
			
				//
				// obter catalogo de produtos
				//
				ds = OleDbHelper.ExecuteDataSet(conn, "SELECT * FROM vwProductCatalog", "Product");
				ds.DataSetName = "Catalog";
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetCatalog> da classe 'Product': " + ex.Message);
				status = ShopStatusEnum.DatabaseError;
			}
			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <GetCatalog> da classe 'Product': " + ex.Message);
				status = ShopStatusEnum.Error;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		}

		#endregion
		
		#region mtodos internos

		/// <summary>
		/// verifica se existe uma determinada quantidade de um produto em armazem.
		/// este mtodo no faz parte do interface pblico da classe, apenas  
		/// utilizado internamente na ClassLibrary
		/// </summary>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="Quant">quantidade desejada de produto</param>
		/// <returns>cdigo de estado relativo  operao</returns></returns></returns>
		internal ShopStatusEnum InStock(OleDbConnection conn, OleDbTransaction tx, int ProductID, int Quant)
		{
			ShopStatusEnum status = ShopStatusEnum.Ok;

			//
			// efectuar o select
			//
			DataSet rs = UtilDB.GetByID(conn, tx, "Products", "ProductID", ProductID, out status);
			if (rs == null)
				return status; //ShopStatusEnum.NotOk;

			//
			// validar stock
			//
			if (rs.Tables["products"].Rows.Count > 0)
				status = ((int)(rs.Tables["Products"].Rows[0]["StockQtd"]) >= Quant ? ShopStatusEnum.Ok : ShopStatusEnum.InsuficientStock);
			else
				status = ShopStatusEnum.InvalidProductID;

			return status;
		}

		#endregion
	}
}
