/*************************************************************

(C) ISEP 2004
(C) Professores de ADAV 2004

O ISEP, na qualidade de autor do presente software, garante 
o direito de utilizao, alterao, evoluo e distribuio 
do mesmo sem qualquer tipo de restries, excepto:
	- esta mensagem no pode ser retirada
	- os nomes das classes no podem ser alterados

*************************************************************/

using System;
using System.Data;
using System.Data.OleDb;



namespace ShopClassLibrary
{
	public interface ISale
	{
		DataSet CreateDetails(string User, string Pass);
		ShopStatusEnum Add(string User, string Pass, int CustomerID, DateTime Date, DataSet RsDetails);
	}

	public interface ISale2 : ISale
	{
		DataSet GetByCustomer(string User, string Pass, int CustomerID);
		DataSet GetAll(string User, string Pass);
	}

	public interface ISale3 : ISale2
	{
		DataSet GetById(string User, string Pass, int saleid);
		ShopStatusEnum GenerateWordDocumentInvoice(string User, string Pass, int saleid, out string filename);
	}

	/// <summary>
	/// Summary description for Sale.
	/// </summary>
	public class Sale : ISale3
	{
		#region ISale

		/// <summary>
		/// pesquisa clientes por nome
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>um dataset vazio para a palicao cliente preencher</returns>
		public DataSet CreateDetails(string User, string Pass)
		{
			OleDbConnection conn = null;
			DataSet ds;
			ShopStatusEnum status = ShopStatusEnum.Ok;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();
				
				//
				// validar o utilizador
				//
				status = UtilDB.ValidateUser(conn, null, User, Pass);
				if (status != ShopStatusEnum.Ok )
					return null;

				//
				// criar o dataset
				//
				ds = UtilDB.GetByID(conn, null, "SaleDetails", "SaleID", -1, out status);;
				if (ds == null)
					return null;

				//
				// configurar as colunas para utilizao
				//
				ds.Tables["SaleDetails"].Columns["SaleID"].AllowDBNull = true;
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <CreateDetails> da classe 'Sale': " + ex.Message);
				status = ShopStatusEnum.DatabaseError;
				return null;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		}

		/// <summary>
		/// adiciona uma venda ao sistema
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="RsDetails">um dataset com as linhas de detalhe da venda</param>
		/// <returns>o cdigo de status relativo  operao</returns>
		public ShopStatusEnum Add(string User, string Pass, int CustomerID, DateTime Date, DataSet RsDetails)
		{
			OleDbConnection conn = null;
			OleDbTransaction tx = null;
			ShopStatusEnum ret = ShopStatusEnum.NotOk;

			//
			// validaes
			//
			if (User.Length == 0 || Pass.Length == 0)
				return ShopStatusEnum.InvalidArgument;
			if (RsDetails == null || RsDetails.Tables.Count != 1 || RsDetails.Tables[0].TableName != "SaleDetails")
				return ShopStatusEnum.InvalidArgument;
			if (RsDetails.Tables["SaleDetails"].Rows.Count == 0)
				return ShopStatusEnum.InvalidArgument;


			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// iniciar transaco
				//
				tx = conn.BeginTransaction();

				//
				// validar o utilizador
				//
				if (UtilDB.ValidateUser(conn, tx, User, Pass) != ShopStatusEnum.Ok)
					return ShopStatusEnum.InvalidLogin;

				//
				// validar cliente
				//
				DataSet dsCust = UtilDB.GetByID(conn, tx, "Customers", "CustomerID", CustomerID, out ret);
				if (ret != ShopStatusEnum.Ok)
					return ret;
				if (dsCust.Tables["Customers"].Rows.Count == 0)
					return ShopStatusEnum.InvalidCustomerID;

				//
				// validar os produtos e quantidades pedidas
				//
//				if (RsDetails.Tables["SaleDetails"].Rows.Count == 0)
//					return ShopStatusEnum.InvalidArgument;

				Product p = new Product();
				int[] listaProd = new int[RsDetails.Tables["SaleDetails"].Rows.Count];
				int i = 0;
				foreach (DataRow regDetails in RsDetails.Tables["SaleDetails"].Rows)
				{
					if (regDetails.IsNull("ProductID"))
						return ShopStatusEnum.InvalidArgument; //INVALID_PRODUCT_ID;
					if (regDetails.IsNull("Quantity"))
						return ShopStatusEnum.InvalidArgument;

					//
					// verificar se este produto tem quantidade suficiente em stock
					// e se existe!
					//
					ret = p.InStock(conn, tx, (int)regDetails["ProductID"], (int)regDetails["Quantity"]);
					if (ret != ShopStatusEnum.Ok)
						return ret;

					//
					// guardar o cdigo de produto para pesquisa
					//
					listaProd[i++] = (int)regDetails["ProductID"];
				}

				//
				// criar um DataSet de produtos e preencher apenas com os produtos desejados
				//
				DataSet pDS = UtilDB.GetByID(conn, tx, "Products", "ProductID", listaProd, out ret);
				if (pDS == null || pDS.Tables[0].Rows.Count != RsDetails.Tables["SaleDetails"].Rows.Count)
					return ShopStatusEnum.InvalidProductID;

				//
				// calcular custo por linha
				//
				foreach (DataRow regDetails in RsDetails.Tables["SaleDetails"].Rows)
				{
					//
					// procurar o produto desejado
					//
					DataRow[] row = pDS.Tables["Products"].Select("ProductId=" + regDetails["ProductID"].ToString());

					//
					// calcular custo desta linha da venda
					//
					regDetails["TotalCost"] = (int)regDetails["Quantity"] * (float)row[0]["UnitPrice"];

					/**
					//
					// actualizar stock no DataSet para posterior update  BD
					//
					row[0]["StockQtd"] = (int)row[0]["StockQtd"] - (int)regDetails["Quantity"];
					**/
				}


				// ======================================
				//
				// tratar das inseres
				//
				// ======================================


				//
				// criar comando para insero
				//
				OleDbCommand cmd = new OleDbCommand("INSERT INTO sales (customerid, dateofsale) VALUES (?, ?)", conn);
				cmd.Transaction = tx;
				cmd.Parameters.Add("custid", CustomerID);
				OleDbParameter parm = cmd.Parameters.Add("dt", OleDbType.Date);
				parm.Value = Date;

				//
				// inserir registo 
				//
				cmd.ExecuteNonQuery();

				//
				// obter id autogerado
				//
				OleDbCommand cmd2 = new OleDbCommand("SELECT @@IDENTITY", conn);
				cmd2.Transaction = tx;
				int id = (int)cmd2.ExecuteScalar();

				//
				// inserir chave estrangeira nas linhas de detalhe
				//
				foreach (DataRow regDetails in RsDetails.Tables["SaleDetails"].Rows)
				{
					regDetails["SaleID"] = id;
				}

				//
				// inserir linhas de detalhe
				//
				OleDbDataAdapter da = new OleDbDataAdapter("SELECT * FROM SaleDetails", conn);
				da.SelectCommand.Transaction = tx;
				OleDbCommandBuilder cb = new OleDbCommandBuilder(da);
				da.Update(RsDetails, "SaleDetails");

				// 
				// actualizar quantidades de produto em stock
				//
				OleDbCommand cmd3 = new OleDbCommand("UPDATE Products SET StockQtd=StockQtd-? WHERE ProductId=?", conn);
				cmd3.Transaction = tx;
				OleDbParameter p1 = cmd3.Parameters.Add("qtd", OleDbType.Integer);
				OleDbParameter p2 = cmd3.Parameters.Add("pid", OleDbType.Integer);
				foreach (DataRow regDetails in RsDetails.Tables["SaleDetails"].Rows)
				{
					p1.Value = regDetails["Quantity"];
					p2.Value = regDetails["ProductID"];

					cmd3.ExecuteNonQuery();
				}

				//
				// fechar transaco
				//
				tx.Commit();
			}
			catch(OleDbException ex)
			{
				//
				// descartar mudanas efectuadas
				//
				if (tx != null)
					tx.Rollback();

				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <Add> da classe 'Sale': " + ex.Message);
				//status = ShopStatusEnum.DatabaseError;
			}
			finally
			{
				conn.Close();
			}

			return ret;
		}
		#endregion

		#region ISale2

		public DataSet GetByCustomer(string User, string Pass, int CustomerID)
		{
			DataSet ds = null;
			OleDbConnection conn = null;

			try
			{				
				//
				// criar objecto DataSet
				//
				ds = new DataSet();

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				if (UtilDB.ValidateUser(conn, null, User, Pass) != ShopStatusEnum.Ok)
					return null;

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// cabealhos das vendas
				//
				string sSqlCmd = "Select * From vwTotalBySale Where CustomerID = ?";
				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Parameters.Add("custid", CustomerID);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, "Sales");

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// linhas de detalhe
				//
				sSqlCmd = "SELECT * FROM vwExpandedSaleDetails WHERE SaleId IN (SELECT SaleID FROM Sales WHERE CustomerId=?)";
				oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Parameters.Add("custid", CustomerID);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, "SaleDetails");

				//
				// criar relao entre tabelas
				//
				ds.Relations.Add("Details", ds.Tables["Sales"].Columns["SaleId"], ds.Tables["SaleDetails"].Columns["SaleId"]);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo 'GetByCustomer': " + ex.Message);
				//status = ShopStatusEnum.DatabaseError;
				ds = null;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;

		}

		public DataSet GetAll(string User, string Pass)
		{
			//
			// validar o utilizador
			//
			if (UtilDB.ValidateUser(User, Pass) != ShopStatusEnum.Ok)
			{
				return null;
			}

			DataSet ds = null;
			OleDbConnection conn = null;

			try
			{				
				//
				// criar objecto DataSet
				//
				ds = new DataSet();

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// cabealhos das vendas
				//
				string sSqlCmd = "Select * From vwTotalBySale";
				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, "Sales");

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// linhas de detalhe
				//
				sSqlCmd = "Select * From vwExpandedSaleDetails";
				oAdapter = new OleDbDataAdapter(sSqlCmd, conn);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, "SaleDetails");

				//
				// criar relao entre tabelas
				//
				ds.Relations.Add("Details", ds.Tables["Sales"].Columns["SaleId"], ds.Tables["SaleDetails"].Columns["SaleId"]);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo 'GetAll': " + ex.Message);
				//status = ShopStatusEnum.DatabaseError;
				ds = null;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;

		}

		#endregion

		#region ISaleOffice Members

		private const string OUTDIR_DOC = @"c:\temp\WinShop\";

		public DataSet GetById(string User, string Pass, int saleid)
		{
			DataSet ds = null;
			OleDbConnection conn = null;

			try
			{				
				//
				// criar objecto DataSet
				//
				ds = new DataSet();

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				conn = new OleDbConnection(UtilDB.CONN);
				conn.Open();

				//
				// validar o utilizador
				//
				if (UtilDB.ValidateUser(conn, null, User, Pass) != ShopStatusEnum.Ok)
					return null;

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// cabealhos das vendas
				//
				string sSqlCmd = "SELECT * FROM vwTotalBySale WHERE SaleId=?";
				OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Parameters.Add("sid", saleid);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, "Sales");

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				// linhas de detalhe
				//
				sSqlCmd = "SELECT * FROM vwExpandedSaleDetails WHERE SaleId=?";
				oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
				oAdapter.SelectCommand.Parameters.Add("sid", saleid);

				//
				// executar o comando e preencher um Dataset
				//
				oAdapter.Fill(ds, "SaleDetails");

				//
				// criar relao entre tabelas
				//
				ds.Relations.Add("Details", ds.Tables["Sales"].Columns["SaleId"], ds.Tables["SaleDetails"].Columns["SaleId"]);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo 'GetSaleById': " + ex.Message);
				//status = ShopStatusEnum.DatabaseError;
				ds = null;
			}
			finally
			{
				//
				// fechar a conexo
				//
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			return ds;
		}

		public ShopStatusEnum GenerateWordDocumentInvoice(string User, string Pass, int saleid, out string filename)
		{
			object missingValue = Type.Missing; 
			object uchar = Word.WdUnits.wdCharacter;
			object uline = Word.WdUnits.wdLine;
			object ucol = Word.WdUnits.wdColumn;
			object i1 = 1;
			object i4 = 4;
			object bfalse = false;
			object btrue = true;

			filename = null;

			// obter factura
			DataSet dsFactura = GetById(User, Pass, saleid);
			if (dsFactura == null)
				return ShopStatusEnum.Error;

			int nFactura = saleid; //devia existir um campo separado para o numero de facturo que tem que ser sequencial e sem "buracos";
			DateTime dataFactura = (DateTime)dsFactura.Tables["Sales"].Rows[0]["DateOfSale"];
			string customerName = (string)dsFactura.Tables["Sales"].Rows[0]["Name"];
			double totalFactura = (double)dsFactura.Tables["Sales"].Rows[0]["SaleTotal"];

			// gerar nome de ficheiro
			object _in_filename = OUTDIR_DOC + "Factura-" + nFactura.ToString() + ".doc";

			// criar objecto Word
			Word.Application w = new Word.ApplicationClass();

			// criar novo documento
			w.Documents.Add(ref missingValue, ref missingValue, ref missingValue, ref missingValue);

			//w.Visible = true;	// s para teste/"debug"!!!!!

			w.Selection.TypeText("Factura n ");
			w.Selection.TypeText(nFactura.ToString());
			w.Selection.TypeParagraph();
			w.Selection.TypeText("Data : ");
			w.Selection.TypeText(dataFactura.ToLongDateString());
			w.Selection.TypeParagraph();
			w.Selection.TypeText("Cliente : ");
			w.Selection.TypeText(customerName);
			w.Selection.TypeParagraph();
			w.Selection.TypeParagraph();

			//
			// criar tabela (cabealho)
			//
			object tb = Word.WdDefaultTableBehavior.wdWord9TableBehavior;
			object afb = Word.WdAutoFitBehavior.wdAutoFitWindow;
			w.ActiveDocument.Tables.Add(w.Selection.Range, 1, 4, ref tb, ref afb);
			
			w.Selection.TypeText("Quantidade");
			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			w.Selection.TypeText("Designao");
			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			w.Selection.TypeText("P.U:");
			w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
			w.Selection.TypeText("sub-total");

			//
			// adicionar as linhas de factura
			//
			foreach (DataRow dr in dsFactura.Tables["SaleDetails"].Rows)
			{
				// adicionar linha  tabela
				w.ActiveDocument.Tables[1].Rows.Add(ref missingValue);
				w.Selection.MoveDown(ref uline, ref i1, ref missingValue);
				w.Selection.MoveLeft(ref uchar, ref i4, ref missingValue);

				w.Selection.TypeText(dr["Quantity"].ToString());
				w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
				w.Selection.TypeText(dr["Description"].ToString());
				w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
				w.Selection.TypeText(dr["UnitPrice"].ToString());
				w.Selection.MoveRight(ref uchar, ref i1, ref missingValue);
				w.Selection.TypeText(dr["TotalCost"].ToString());
			}
			w.Selection.MoveDown(ref uline, ref i1, ref missingValue);

			w.Selection.TypeParagraph();
			w.Selection.TypeText("Total :");
			w.Selection.TypeText(totalFactura.ToString());
			w.Selection.TypeParagraph();

			//
			// formatar tabela
			//
			object tf = Word.WdTableFormat.wdTableFormatProfessional;
			w.ActiveDocument.Tables[1].AutoFormat(ref tf, ref btrue, ref btrue, ref btrue, ref btrue, ref btrue, ref bfalse, ref bfalse, ref bfalse, ref bfalse);

			// rodap da factura
			w.Selection.TypeParagraph();
			w.Selection.TypeText("Generated by WinShop 1.1");

			// gravar ficheiro
			w.ActiveDocument.SaveAs(ref _in_filename, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue, ref missingValue);

			//terminar Word
			w.Quit(ref missingValue, ref missingValue, ref missingValue);


			filename = (string)_in_filename;
			return ShopStatusEnum.Ok;
		}

		#endregion
	}
}
